'|==========================================================================
'|                    3D Mandelbrot Fractal
'|                  (c)2001 by Maxim Kakitsev
'|                      maksim@freeuk.com
'|==========================================================================

' This program draws Mandelbrot Set in 3D, reading data from the file 3DM.dat
' You can enter horizontal and vertical angles at which to view this fractal.
' You can also choose the distance (the depth) into the screen.
' It's better to keep the horizontal angle at either 1 or 180, other values
' for this angle might not work properly.
' Experiment with different angles to figure out how it works.
'
' It's better to start with these values:
'        horizontal angle = 180
'        vertical angle = 340
'        distance into the screen = 500
'
' Try this for an interesting "landscape" view:
'        horizontal = 180
'        vertical = 150
'        distance = 250

SCREEN 12

DIM S!(540)
DIM C!(540)
PI = 3.141592

FOR i = 0 TO 359
   S!(i) = SIN(i * (PI / 180))
   C!(i) = COS(i * (PI / 180))
NEXT

start:
CLS
    '! Change the path below to the correct one !'
'                 /
'               /
OPEN "c:\3DMand\3DM.dat" FOR INPUT AS #1

PRINT "Enter horizontal angle (between 0 and 360)": INPUT theta%
IF theta% >= 360 THEN theta% = theta% - 360  'just in case you enter more :-)
PRINT "vertical angle (between 0 and 360)": INPUT phi%
IF phi% >= 360 THEN phi% = phi% - 360
PRINT "distance into the screen (100 to 1000)": INPUT zcenter

'You can change the values below to move the image sideways:
xcenter = 320: ycenter = 240

CLS

FOR a = 1 TO 15        'fast palette (the PALETTE statement is slow, people!)
  OUT &H3C8, a                 'standard color number
  OUT &H3C9, 0                 'RED
  OUT &H3C9, a * 4 + 3         'GREEN
  OUT &H3C9, 63 - (a * 4 + 3)  'BLUE
NEXT a

DO UNTIL EOF(1)
   INPUT #1, x3, y3, z3
   IF z3 < 75 OR z3 + zcenter > 20 THEN
     xr = -x3 * S!(theta%) + y3 * C!(theta%)
     yr = -x3 * C!(theta%) * S!(phi%) - y3 * S!(theta%) * S!(phi%) - z3 * C!(phi%)
     zr = -x3 * C!(theta%) * C!(phi%) - y3 * S!(theta%) * C!(phi%) + z3 * S!(phi%)
scrX = 256 * (xr / (zr + zcenter)) + xcenter
scrY = 256 * (yr / (zr + zcenter)) + ycenter
  l = z3
  IF l = 75 THEN l = 0
  IF l > 60 THEN l = l - 59
  IF l > 45 THEN l = l - 44
  IF l > 30 THEN l = l - 29
  IF l > 15 THEN l = l - 14
  PSET (scrX, scrY), l
  END IF
LOOP
CLOSE #1

FOR change = 63 TO 126      'well, I used PALETTE here to have a nice
FOR pal = 1 TO 15           'slow color change
  bl = 63 - (pal * 4 + 3)
  gr = (pal * 4 + 3)
  red = change - gr - bl
  PALETTE pal, red + gr * 256 + bl * 256 ^ 2
NEXT pal
NEXT change

INPUT "Press 'Y' to change the angles, or any other key to quit.", ch$
IF UCASE$(ch$) = "Y" THEN GOTO start
END

